#! /usr/bin/python

import serial
from pylab import *
import sys
import string


def thd(x):
    n = len(x)
    X = abs(fft(x)) / n
    X1rms2 = 2 * X[1] ** 2
    Xrms2 = sum(X ** 2)
    return 100 * sqrt(Xrms2 / X1rms2 - 1)
    
def rms(x):
    return sqrt(mean(x ** 2))
    
def apower(x, y):
    return mean(x * y)

def dpf(x, y):
    X = fft(x)
    Y = fft(y)
    return cos(angle(X[1]) - angle(Y[1]))



# Let's start here

whatfilter = False
whatcurrent = False
whattotake = False
par = sys.argv
if len(par) > 1:
    m_type = string.upper(par[1][0])
    if m_type in ['L', 'C', 'N']:
        whatfilter = True
    if len(par[1]) > 1:
        m_out = string.upper(par[1][1])
        if m_out in ['1', '2', '3', '4', '5']:
            whatcurrent = True
    if len(par[1]) > 2:
        phase = string.lower(par[1][2:])
        if phase in ['1', '2', '3', 'out']:
            whattotake = True

if whatfilter == False:
    while True:
        m_type = raw_input('filter type (L, C or N): ')
        m_type = string.upper(m_type)
        if m_type in ['L', 'C', 'N']:
            break
        else:
            print 'wrong input, try again'

if whatcurrent == False:
    while True:
        m_out = raw_input('output current in [A] (1, 2, 3, 4, or 5): ')
        if m_out in ['1', '2', '3', '4', '5']:
            break
        else:
            print 'wrong input, try again'

if whattotake == False:
    while True:
        phase = raw_input('phase # or output (1, 2, 3, or out): ')
        if phase in ['1', '2', '3', 'out']:
            break
        else:
            print 'wrong input, try again'



c_probe = 0.1 # V/A

s = serial.Serial(0)
s.timeout = 5

s.write('CH1:COUPling DC;BANdwidth ON;PRObe 1;POSition 0\n')
s.write('CH2:COUPling DC;BANdwidth ON;PRObe 100;POSition 0\n')

s.write('ACQuire:MODe SAMPLE\n')

s.write('TRIGger:MAIn:EDGE:SOUrce LINE\n')
s.write('TRIGger:MAIn:EDGE:SLOpe RISE\n')

s.write('HORizontal:MAIn:SCAle 2.5E-3\n')
s.write('HORizontal:MAIn:POSition 0\n')

v_position = 0.0
h_scale = 2.5e-3
h_position = 0



print 'taking current . . .'

ch = 1
sch = str(ch)
s.write('CH' + sch + ':SCALe?\n')
v_scale = float(s.readline())
s.write('DATa:SOUrce CH' + sch + '\n')
s.write('DATa:ENCdg ASCII\n')
s.write('DATa:STARt 251\n')
s.write('DATa:STOP 2250\n')
s.write('CURVe?\n')
datastring = s.readline()
datalist = datastring.split(',')
ch1 = []
for i in datalist:
    ch1.append((float(i) / 25 - v_position) * v_scale)
ch1 = array(ch1) / c_probe
cmin = ((- 4 - v_position) * v_scale) / c_probe
cmax = ((4 - v_position) * v_scale) / c_probe



print 'taking voltage . . .'

ch = 2
sch = str(ch)
s.write('CH' + sch + ':SCALe?\n')
v_scale = float(s.readline())
s.write('DATa:ENCdg ASCII\n')
s.write('DATa:STARt 251\n')
s.write('DATa:STOP 2250\n')
s.write('DATa:SOUrce CH' + sch + '\n')
s.write('CURVe?\n')
datastring = s.readline()
datalist = datastring.split(',')
ch2 = []
for i in datalist:
    ch2.append((float(i) / 25 - v_position) * v_scale)
ch2 = array(ch2)
vmin = (- 4 - v_position) * v_scale
vmax = (4 - v_position) * v_scale

s.close()


print 'computing . . .'

t = (arange(-4, 4, 0.004) * h_scale + h_position) * 1e3

#dat=open(m_type + m_out + 'ph' + phase + '.csv','w')
#for i in range(len(ch1)):
#    dat.write(str(ch1[i]) + ',' + str(ch2[i]) + '\n')
#dat.close()

np.save(m_type + '/' + m_type + m_out + 'i' + phase + '.npy', ch1)
np.save(m_type + '/' + m_type + m_out + 'v' + phase + '.npy', ch2)

if phase != 'out':

    thdi = thd(ch1)
    thdv = thd(ch2)
    P = apower(ch1, ch2)
    Irms = rms(ch1)
    Vrms = rms(ch2)
    S = Irms * Vrms
    PF = P / S
    DPF = dpf(ch1, ch2)

    print 'THD(i' + phase + ') = {:.2f} %'.format(thdi)
    print 'THD(v' + phase + ') = {:.2f} %'.format(thdv)
    print 'Pin' + phase + ' = {:.2f} W'.format(P)
    print 'Irms' + phase + ' = {:.2f} A'.format(Irms)
    print 'Vrms' + phase + ' = {:.2f} V'.format(Vrms)
    print 'S' + phase + ' = {:.2f} VA'.format(S)
    print 'PF' + phase + ' = {:.4f}'.format(PF)
    print 'DPF' + phase + ' = {:.4f}'.format(DPF)

else:

    Pout = apower(ch1, ch2)
    Iout = mean(ch1)
    Vout = mean(ch2)

    print 'Pout = {:.2f} W'.format(Pout)
    print 'Iout = {:.2f} A'.format(Iout)
    print 'Vout = {:.2f} V'.format(Vout)


fig=figure(1)

fc = fig.add_subplot(2, 1, 1)
fc.plot(t, ch1)
fc.axis([-10, 10, cmin, cmax])
fc.set_xlabel('time [ms]')
fc.set_ylabel('i' + phase + ' [A]')

fv = fig.add_subplot(2, 1, 2)
fv.plot(t, ch2)
fv.axis([-10, 10, vmin, vmax])
fv.set_xlabel('time [ms]')
fv.set_ylabel('v' + phase + ' [V]')

show()

